@@ -38,8 +38,13 @@
 #include "fr-command.h"
 #include "fr-error.h"
 #include "fr-marshal.h"
+#include "fr-proc-error.h"
 #include "fr-process.h"
-#include "main.h"
+#include "fr-init.h"
+
+#if ENABLE_MAGIC
+#include <magic.h>
+#endif
 
 #ifndef NCARGS
 #define NCARGS _POSIX_ARG_MAX
@@ -111,9 +116,6 @@ struct _FrArchivePrivData {
 							     * fr_archive_load is invoked, used
 							     * in batch mode. */
 	gpointer             fake_load_data;
-	FakeLoadFunc         add_is_stoppable_func;         /* Returns whether the add operation is
-							     * stoppable. */
-	gpointer             add_is_stoppable_data;
 	GCancellable        *cancellable;
 	char                *temp_dir;
 	gboolean             continue_adding_dropped_items;
@@ -246,10 +248,10 @@ fr_archive_class_init (FrArchiveClass *class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (FrArchiveClass, done),
 			      NULL, NULL,
-			      fr_marshal_VOID__INT_POINTER,
+			      fr_marshal_VOID__INT_BOXED,
 			      G_TYPE_NONE, 2,
 			      G_TYPE_INT,
-			      G_TYPE_POINTER);
+			      FR_TYPE_PROC_ERROR);
 	fr_archive_signals[PROGRESS] =
 		g_signal_new ("progress",
 			      G_TYPE_FROM_CLASS (class),
@@ -334,21 +336,11 @@ fr_archive_action_completed (FrArchive       *archive,
 }
 
 
-static gboolean
-fr_archive_add_is_stoppable (FrArchive *archive)
-{
-	if (archive->priv->add_is_stoppable_func != NULL)
-		return (*archive->priv->add_is_stoppable_func) (archive, archive->priv->add_is_stoppable_data);
-	else
-		return FALSE;
-}
-
-
 static gboolean
 archive_sticky_only_cb (FrProcess *process,
 			FrArchive *archive)
 {
-	fr_archive_stoppable (archive, fr_archive_add_is_stoppable (archive));
+	fr_archive_stoppable (archive, FALSE);
 	return TRUE;
 }
 
@@ -366,8 +358,6 @@ fr_archive_init (FrArchive *archive)
 	archive->priv = g_new0 (FrArchivePrivData, 1);
 	archive->priv->fake_load_func = NULL;
 	archive->priv->fake_load_data = NULL;
-	archive->priv->add_is_stoppable_func = NULL;
-	archive->priv->add_is_stoppable_data = NULL;
 
 	archive->priv->extraction_destination = NULL;
 	archive->priv->temp_extraction_dir = NULL;
@@ -525,43 +515,72 @@ get_mime_type_from_content (GFile *file)
 static const char *
 get_mime_type_from_magic_numbers (GFile *file)
 {
-	static struct {
-		const char *mime_type;
-		const char *first_bytes;
-		int         offset;
-		int         len;
-	} sniffer_data [] = {
-		/* Magic numbers taken from magic/Magdir/archive from the
-		 * file-4.21 tarball. */
-		{ "application/x-7z-compressed", "7z\274\257\047\034", 0, 5 },
-		{ "application/x-ace", "**ACE**", 7, 7 },
-		{ "application/x-arj", "\x60\xea", 0, 2 },
-		{ "application/x-bzip2", "BZh", 0, 3 },
-		{ "application/x-gzip", "\037\213", 0, 2 },
-		{ "application/x-lzip", "LZIP", 0, 4 },
-		{ "application/x-lzop", "\x89\x4c\x5a\x4f\x00\x0d\x0a\x1a\x0a", 0, 9 },
-		{ "application/x-rar", "Rar!", 0, 4 },
-		{ "application/x-rzip", "RZIP", 0, 4 },
-		{ "application/x-xz", "\3757zXZ\000", 0, 6 },
-		{ "application/x-zoo", "\xdc\xa7\xc4\xfd", 20, 4 },
-		{ "application/zip", "PK\003\004", 0, 4 },
-		{ "application/zip", "PK00PK\003\004", 0, 8 },
-		{ "application/x-lrzip", "LRZI", 0, 4 },
-		{ NULL, NULL, 0 }
+#if ENABLE_MAGIC
+	static magic_t magic = NULL;
+
+	if (! magic) {
+		magic = magic_open (MAGIC_MIME_TYPE);
+		if (magic)
+			magic_load (magic, NULL);
+		else
+			g_warning ("unable to open magic database");
+	}
+
+	if (magic) {
+		const char * mime_type = NULL;
+		char * filepath = g_file_get_path (file);
+
+		if (filepath) {
+			mime_type = magic_file (magic, filepath);
+			g_free (filepath);
+		}
+
+		if (mime_type)
+			return mime_type;
+
+		g_warning ("unable to detect filetype from magic: %s",
+			   magic_error (magic));
+	}
+#else
+	static const struct magic {
+		const unsigned int off;
+		const unsigned int len;
+		const char * const id;
+		const char * const mime_type;
+	} magic_ids [] = {
+		/* magic ids taken from magic/Magdir/archive from the file-4.21 tarball */
+		{ 0,  6, "7z\274\257\047\034",                   "application/x-7z-compressed" },
+		{ 7,  7, "**ACE**",                              "application/x-ace"           },
+		{ 0,  2, "\x60\xea",                             "application/x-arj"           },
+		{ 0,  3, "BZh",                                  "application/x-bzip2"         },
+		{ 0,  2, "\037\213",                             "application/x-gzip"          },
+		{ 0,  4, "LZIP",                                 "application/x-lzip"          },
+		{ 0,  9, "\x89\x4c\x5a\x4f\x00\x0d\x0a\x1a\x0a", "application/x-lzop",         },
+		{ 0,  4, "Rar!",                                 "application/x-rar"           },
+		{ 0,  4, "RZIP",                                 "application/x-rzip"          },
+		{ 0,  6, "\3757zXZ\000",                         "application/x-xz"            },
+		{ 20, 4, "\xdc\xa7\xc4\xfd",                     "application/x-zoo",          },
+		{ 0,  4, "PK\003\004",                           "application/zip"             },
+		{ 0,  8, "PK00PK\003\004",                       "application/zip"             },
+		{ 0,  4, "LRZI",                                 "application/x-lrzip"         },
 	};
+
 	char buffer[32];
 	int  i;
 
-	if (! g_load_file_in_buffer (file, buffer, 32, NULL))
+	if (! g_load_file_in_buffer (file, buffer, sizeof (buffer), NULL))
 		return NULL;
 
-	for (i = 0; sniffer_data[i].mime_type != NULL; i++)
-		if (memcmp (sniffer_data[i].first_bytes,
-			    buffer + sniffer_data[i].offset,
-			    sniffer_data[i].len) == 0)
-		{
-			return sniffer_data[i].mime_type;
-		}
+	for (i = 0; i < G_N_ELEMENTS (magic_ids); i++) {
+		const struct magic * const magic = &magic_ids[i];
+
+		if (sizeof (buffer) < (magic->off + magic->len))
+			g_warning ("buffer underrun for mime type '%s' magic",
+				   magic->mime_type);
+		else if (! memcmp (buffer + magic->off, magic->id, magic->len))
+			return magic->mime_type;
+	}
+#endif
 
 	return NULL;
 }
@@ -590,18 +609,13 @@ create_command_from_type (FrArchive     *archive,
 		          GType          command_type,
 		          FrCommandCaps  requested_capabilities)
 {
-	char *filename;
-
 	if (command_type == 0)
 		return FALSE;
 
-	filename = g_file_get_path (archive->local_copy);
 	archive->command = FR_COMMAND (g_object_new (command_type,
 					             "process", archive->process,
-					             "filename", filename,
 					             "mime-type", mime_type,
 					             NULL));
-	g_free (filename);
 
 	if (! fr_command_is_capable_of (archive->command, requested_capabilities)) {
 		g_object_unref (archive->command);
@@ -1116,10 +1130,10 @@ static void
 load_local_archive (FrArchive  *archive,
 		    const char *password)
 {
-	FrCommand  *tmp_command;
+	FrCommand  *old_command;
 	const char *mime_type;
 
-	if (! g_file_query_exists (archive->file, NULL)) {
+	if (! g_file_query_exists (archive->file, archive->priv->cancellable)) {
 		fr_archive_action_completed (archive,
 					     FR_ACTION_LOADING_ARCHIVE,
 					     FR_PROC_ERROR_GENERIC,
@@ -1130,7 +1144,7 @@ load_local_archive (FrArchive  *archive,
 	archive->have_permissions = check_file_permissions (archive->file, W_OK);
 	archive->read_only = ! archive->have_permissions;
 
-	tmp_command = archive->command;
+	old_command = archive->command;
 
 	mime_type = get_mime_type_from_filename (archive->local_copy);
 	if (! create_command_to_load_archive (archive, mime_type)) {
@@ -1138,7 +1152,7 @@ load_local_archive (FrArchive  *archive,
 		if (! create_command_to_load_archive (archive, mime_type)) {
 			mime_type = get_mime_type_from_magic_numbers (archive->local_copy);
 			if (! create_command_to_load_archive (archive, mime_type)) {
-				archive->command = tmp_command;
+				archive->command = old_command;
 				archive->content_type = mime_type;
 				fr_archive_action_completed (archive,
 							     FR_ACTION_LOADING_ARCHIVE,
@@ -1149,9 +1163,9 @@ load_local_archive (FrArchive  *archive,
 		}
 	}
 
-	if (tmp_command != NULL) {
-		g_signal_handlers_disconnect_by_data (tmp_command, archive);
-		g_object_unref (tmp_command);
+	if (old_command != NULL) {
+		g_signal_handlers_disconnect_by_data (old_command, archive);
+		g_object_unref (old_command);
 	}
 
 	fr_archive_connect_to_command (archive);
@@ -1169,9 +1183,11 @@ load_local_archive (FrArchive  *archive,
 	/**/
 
 	fr_process_clear (archive->process);
-	g_object_set (archive->command, "password", password, NULL);
+	g_object_set (archive->command,
+		      "file", archive->local_copy,
+		      "password", password,
+		      NULL);
 	fr_command_list (archive->command);
-	fr_process_start (archive->process);
 }
 
 
@@ -1224,11 +1240,13 @@ copy_remote_file (FrArchive  *archive,
 {
 	XferData *xfer_data;
 
-	if (! g_file_query_exists (archive->file, NULL)) {
+	if (! g_file_query_exists (archive->file, archive->priv->cancellable)) {
 		GError *error;
-		error = g_error_new (G_IO_ERROR, G_IO_ERROR_NOT_FOUND, _("The file doesn't exist"));
+
+		error = g_error_new (G_IO_ERROR, G_IO_ERROR_NOT_FOUND, _("Archive not found"));
 		fr_archive_copy_done (archive, FR_ACTION_LOADING_ARCHIVE, error);
 		g_error_free (error);
+
 		return;
 	}
 
@@ -1311,27 +1329,6 @@ fr_archive_reload (FrArchive  *archive,
 }
 
 
-void
-fr_archive_rename (FrArchive  *archive,
-		   const char *filename)
-{
-	g_return_if_fail (archive != NULL);
-
-	if (archive->is_compressed_file) {
-		/* If the archive is a compressed file we have to reload it,
-		 * because in this case the 'content' of the archive changes
-		 * too. */
-		fr_archive_load (archive, filename, NULL);
-	}
-	else {
-		if (archive->file != NULL)
-			g_object_unref (archive->file);
-		archive->file = g_file_new_for_path (filename);
-		fr_command_set_filename (archive->command, filename);
-	}
-}
-
-
 /* -- add -- */
 
 
@@ -1437,16 +1434,6 @@ newer_files_only (FrArchive  *archive,
 	return newer_files;
 }
 
-
-void
-fr_archive_set_add_is_stoppable_func (FrArchive     *archive,
-				      FakeLoadFunc   func,
-				      gpointer       data)
-{
-	archive->priv->add_is_stoppable_func = func;
-	archive->priv->add_is_stoppable_data = data;
-}
-
 static gboolean
 save_list_to_temp_file (GList   *file_list,
 		        char   **list_dir,
@@ -1554,6 +1541,9 @@ fr_archive_add (FrArchive     *archive,
 	gboolean  base_dir_created = FALSE;
 	GList    *scan;
 	char     *tmp_base_dir = NULL;
+	char     *tmp_archive_dir = NULL;
+	char     *archive_filename = NULL;
+	char     *tmp_archive_filename = NULL;
 	gboolean  error_occurred = FALSE;
 
 	if (file_list == NULL)
@@ -1569,9 +1559,9 @@ fr_archive_add (FrArchive     *archive,
 		      "volume_size", volume_size,
 		      NULL);
 
-	fr_archive_stoppable (archive, fr_archive_add_is_stoppable (archive));
+	fr_archive_stoppable (archive, TRUE);
 
-	tmp_base_dir = g_strdup (base_dir);
+	/* dest_dir is the destination folder inside the archive */
 
 	if ((dest_dir != NULL) && (*dest_dir != '\0') && (strcmp (dest_dir, "/") != 0)) {
 		const char *rel_dest_dir = dest_dir;
@@ -1588,8 +1578,10 @@ fr_archive_add (FrArchive     *archive,
 			new_file_list = g_list_prepend (new_file_list, g_build_filename (rel_dest_dir, filename, NULL));
 		}
 	}
-	else
+	else {
+		tmp_base_dir = g_strdup (base_dir);
 		new_file_list = path_list_dup(file_list);
+	}
 
 	/* if the command cannot update,  get the list of files that are
 	 * newer than the ones in the archive. */
@@ -1612,11 +1604,46 @@ fr_archive_add (FrArchive     *archive,
 		archive->process->error.type = FR_PROC_ERROR_NONE;
 		g_signal_emit_by_name (G_OBJECT (archive->process),
 				       "done",
-				       FR_ACTION_ADDING_FILES);
+				       &archive->process->error);
 		return;
 	}
 
-	archive->command->creating_archive = ! g_file_test (archive->command->filename, G_FILE_TEST_EXISTS);
+	archive->command->creating_archive = ! g_file_query_exists (archive->local_copy, archive->priv->cancellable);
+
+	/* create the new archive in a temporary sub-directory, this allows
+	 * to cancel the operation without losing the original archive and
+	 * removing possible temporary files created by the command. */
+
+	{
+		GFile *local_copy_parent;
+		char  *archive_dir;
+		GFile *tmp_file;
+
+		/* create the new archive in a sub-folder of the original
+		 * archive this way the 'mv' command is fast. */
+
+		local_copy_parent = g_file_get_parent (archive->local_copy);
+		archive_dir = g_file_get_path (local_copy_parent);
+		tmp_archive_dir = get_temp_work_dir (archive_dir);
+		archive_filename = g_file_get_path (archive->local_copy);
+		tmp_archive_filename = g_build_filename (tmp_archive_dir, file_name_from_path (archive_filename), NULL);
+		tmp_file = g_file_new_for_path (tmp_archive_filename);
+		g_object_set (archive->command, "file", tmp_file, NULL);
+
+		if (! archive->command->creating_archive) {
+			/* copy the original archive to the new position */
+
+			fr_process_begin_command (archive->process, "cp");
+			fr_process_add_arg (archive->process, "-f");
+			fr_process_add_arg (archive->process, archive_filename);
+			fr_process_add_arg (archive->process, tmp_archive_filename);
+			fr_process_end_command (archive->process);
+		}
+
+		g_object_unref (tmp_file);
+		g_free (archive_dir);
+		g_object_unref (local_copy_parent);
+	}
 
 	fr_command_uncompress (archive->command);
 
@@ -1664,7 +1691,7 @@ fr_archive_add (FrArchive     *archive,
 			archive->process->error.gerror = g_error_copy (error);
 			g_signal_emit_by_name (G_OBJECT (archive->process),
 					       "done",
-					       FR_ACTION_ADDING_FILES);
+					       &archive->process->error);
 			g_clear_error (&error);
 			error_occurred = TRUE;
 		}
@@ -1717,7 +1744,26 @@ fr_archive_add (FrArchive     *archive,
 	if (! error_occurred) {
 		fr_command_recompress (archive->command);
 
-		if (base_dir_created) { /* remove the temp dir */
+		/* move the new archive to the original position */
+
+		fr_process_begin_command (archive->process, "mv");
+		fr_process_add_arg (archive->process, "-f");
+		fr_process_add_arg (archive->process, tmp_archive_filename);
+		fr_process_add_arg (archive->process, archive_filename);
+		fr_process_end_command (archive->process);
+
+		/* remove the temp sub-directory */
+
+		fr_process_begin_command (archive->process, "rm");
+		fr_process_set_working_dir (archive->process, g_get_tmp_dir());
+		fr_process_set_sticky (archive->process, TRUE);
+		fr_process_add_arg (archive->process, "-rf");
+		fr_process_add_arg (archive->process, tmp_archive_dir);
+		fr_process_end_command (archive->process);
+
+		/* remove the base dir */
+
+		if (base_dir_created) {
 			fr_process_begin_command (archive->process, "rm");
 			fr_process_set_working_dir (archive->process, g_get_tmp_dir());
 			fr_process_set_sticky (archive->process, TRUE);
@@ -1727,6 +1773,9 @@ fr_archive_add (FrArchive     *archive,
 		}
 	}
 
+	g_free (tmp_archive_filename);
+	g_free (archive_filename);
+	g_free (tmp_archive_dir);
 	g_free (tmp_base_dir);
 }
 
@@ -1742,7 +1791,6 @@ fr_archive_add_local_files (FrArchive     *archive,
 			    FrCompression  compression,
 			    guint          volume_size)
 {
-	fr_archive_stoppable (archive, TRUE);
 	fr_process_clear (archive->process);
 	fr_archive_add (archive,
 			file_list,
@@ -2341,7 +2389,9 @@ add_dropped_items (DroppedItemsData *data)
 	 * files without path info. FIXME: doesn't work with remote files. */
 
 	fr_archive_stoppable (archive, FALSE);
+	archive->command->creating_archive = ! g_file_query_exists (archive->local_copy, archive->priv->cancellable);
 	g_object_set (archive->command,
+		      "file", archive->local_copy,
 		      "password", data->password,
 		      "encrypt_header", data->encrypt_header,
 		      "compression", data->compression,
@@ -2567,16 +2617,80 @@ fr_archive_remove (FrArchive     *archive,
 		   GList         *file_list,
 		   FrCompression  compression)
 {
+	char *tmp_archive_dir = NULL;
+	char *archive_filename = NULL;
+	char *tmp_archive_filename = NULL;
+
 	g_return_if_fail (archive != NULL);
 
 	if (archive->read_only)
 		return;
 
-	fr_archive_stoppable (archive, FALSE);
+	fr_archive_stoppable (archive, TRUE);
+	archive->command->creating_archive = FALSE;
 	g_object_set (archive->command, "compression", compression, NULL);
+
+	/* create the new archive in a temporary sub-directory, this allows
+	 * to cancel the operation without losing the original archive and
+	 * removing possible temporary files created by the command. */
+
+	{
+		GFile *local_copy_parent;
+		char  *archive_dir;
+		GFile *tmp_file;
+
+		/* create the new archive in a sub-folder of the original
+		 * archive this way the 'mv' command is fast. */
+
+		local_copy_parent = g_file_get_parent (archive->local_copy);
+		archive_dir = g_file_get_path (local_copy_parent);
+		tmp_archive_dir = get_temp_work_dir (archive_dir);
+		archive_filename = g_file_get_path (archive->local_copy);
+		tmp_archive_filename = g_build_filename (tmp_archive_dir, file_name_from_path (archive_filename), NULL);
+		tmp_file = g_file_new_for_path (tmp_archive_filename);
+		g_object_set (archive->command, "file", tmp_file, NULL);
+
+		if (! archive->command->creating_archive) {
+			/* copy the original archive to the new position */
+
+			fr_process_begin_command (archive->process, "cp");
+			fr_process_add_arg (archive->process, "-f");
+			fr_process_add_arg (archive->process, archive_filename);
+			fr_process_add_arg (archive->process, tmp_archive_filename);
+			fr_process_end_command (archive->process);
+		}
+
+		g_object_unref (tmp_file);
+		g_free (archive_dir);
+		g_object_unref (local_copy_parent);
+	}
+
+	/* uncompress, delete and recompress */
+
 	fr_command_uncompress (archive->command);
 	delete_from_archive (archive, file_list);
 	fr_command_recompress (archive->command);
+
+	/* move the new archive to the original position */
+
+	fr_process_begin_command (archive->process, "mv");
+	fr_process_add_arg (archive->process, "-f");
+	fr_process_add_arg (archive->process, tmp_archive_filename);
+	fr_process_add_arg (archive->process, archive_filename);
+	fr_process_end_command (archive->process);
+
+	/* remove the temp sub-directory */
+
+	fr_process_begin_command (archive->process, "rm");
+	fr_process_set_working_dir (archive->process, g_get_tmp_dir());
+	fr_process_set_sticky (archive->process, TRUE);
+	fr_process_add_arg (archive->process, "-rf");
+	fr_process_add_arg (archive->process, tmp_archive_dir);
+	fr_process_end_command (archive->process);
+
+	g_free (tmp_archive_filename);
+	g_free (archive_filename);
+	g_free (tmp_archive_dir);
 }
 
 
@@ -2942,6 +3056,7 @@ fr_archive_extract_to_local (FrArchive  *archive,
 	g_return_if_fail (archive != NULL);
 
 	fr_archive_stoppable (archive, TRUE);
+	g_object_set (archive->command, "file", archive->local_copy, NULL);
 
 	/* if a command supports all the requested options use
 	 * fr_command_extract directly. */
@@ -3295,7 +3410,10 @@ fr_archive_test (FrArchive  *archive,
 {
 	fr_archive_stoppable (archive, TRUE);
 
-	g_object_set (archive->command, "password", password, NULL);
+	g_object_set (archive->command,
+		      "file", archive->local_copy,
+		      "password", password,
+		      NULL);
 	fr_process_clear (archive->process);
 	fr_command_set_n_files (archive->command, 0);
 	fr_command_test (archive->command);
