@@ -34,6 +34,7 @@
 #include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-7z.h"
+#include "rar-utils.h"
 
 static void fr_command_7z_class_init  (FrCommand7zClass *class);
 static void fr_command_7z_init        (FrCommand        *afile);
@@ -159,7 +160,7 @@ list__process_line (char     *line,
 		fdata->original_path = g_strdup (fields[1]);
 		fdata->full_path = g_strconcat ((fdata->original_path[0] != '/') ? "/" : "",
 						fdata->original_path,
-						(fdata->dir && (fdata->original_path[strlen (fdata->original_path - 1)] != '/')) ? "/" : "",
+						(fdata->dir && (fdata->original_path[strlen (fdata->original_path) - 1] != '/')) ? "/" : "",
 						NULL);
 	}
 	else if (strcmp (fields[0], "Folder") == 0) {
@@ -235,6 +236,8 @@ list__begin (gpointer data)
 static void
 fr_command_7z_list (FrCommand  *comm)
 {
+	rar_check_multi_volume (comm);
+
 	fr_process_set_out_line_func (comm->process, list__process_line, comm);
 
 	fr_command_7z_begin_command (comm);
@@ -252,10 +255,6 @@ fr_command_7z_list (FrCommand  *comm)
 }
 
 
-static char Progress_Message[4196];
-static char Progress_Filename[4096];
-
-
 static void
 parse_progress_line (FrCommand  *comm,
 		     const char *prefix,
@@ -265,16 +264,8 @@ parse_progress_line (FrCommand  *comm,
 	int prefix_len;
 
 	prefix_len = strlen (prefix);
-	if (strncmp (line, prefix, prefix_len) == 0) {
-		double fraction;
-
-		strcpy (Progress_Filename, line + prefix_len);
-		sprintf (Progress_Message, "%s%s", message_prefix, file_name_from_path (Progress_Filename));
-		fr_command_message (comm, Progress_Message);
-
-		fraction = (double) ++comm->n_file / (comm->n_files + 1);
-		fr_command_progress (comm, fraction);
-	}
+	if (strncmp (line, prefix, prefix_len) == 0)
+		fr_command_progress (comm, (double) ++comm->n_file / (comm->n_files + 1));
 }
 
 
@@ -285,10 +276,14 @@ process_line__add (char     *line,
 	FrCommand *comm = FR_COMMAND (data);
 
 	if ((comm->volume_size > 0) && (strncmp (line, "Creating archive ", 17) == 0)) {
-		char *volume_filename;
+		char  *volume_filename;
+		GFile *volume_file;
 
 		volume_filename = g_strconcat (comm->filename, ".001", NULL);
-		fr_command_set_multi_volume (comm, volume_filename);
+		volume_file = g_file_new_for_path (volume_filename);
+		fr_command_set_multi_volume (comm, volume_file);
+
+		g_object_unref (volume_file);
 		g_free (volume_filename);
 	}
 
@@ -537,14 +532,14 @@ const char *sevenz_mime_types[] = { "application/x-7z-compressed",
 				    NULL };
 
 
-const char **
+static const char **
 fr_command_7z_get_mime_types (FrCommand *comm)
 {
 	return sevenz_mime_types;
 }
 
 
-FrCommandCap
+static FrCommandCap
 fr_command_7z_get_capabilities (FrCommand  *comm,
 				const char *mime_type,
 				gboolean    check_command)
