@@ -35,6 +35,7 @@
 #include "fr-command.h"
 #include "fr-command-rar.h"
 #include "fr-error.h"
+#include "rar-utils.h"
 
 static void fr_command_rar_class_init  (FrCommandRarClass *class);
 static void fr_command_rar_init        (FrCommand         *afile);
@@ -151,6 +152,9 @@ parse_name_field (char         *line,
 	else
 		name_field = g_strdup (line + 1);
 
+	if (name_field == NULL)
+		return;
+
 	if (*name_field == '/') {
 		fdata->full_path = g_strdup (name_field);
 		fdata->original_path = fdata->full_path;
@@ -312,115 +316,6 @@ add_password_arg (FrCommand  *comm,
 }
 
 
-typedef enum {
-	FIRST_VOLUME_IS_000,
-	FIRST_VOLUME_IS_001,
-	FIRST_VOLUME_IS_RAR
-} FirstVolumeExtension;
-
-
-static char *
-get_first_volume_name (const char           *name,
-		       const char           *pattern,
-		       FirstVolumeExtension  extension_type)
-{
-	char   *volume_name = NULL;
-	GRegex *re;
-
-	re = g_regex_new (pattern, G_REGEX_CASELESS, 0, NULL);
-	if (g_regex_match (re, name, 0, NULL)) {
-		char **parts;
-		int    l, i;
-
-		parts = g_regex_split (re, name, 0);
-		l = strlen (parts[2]);
-		switch (extension_type) {
-		case FIRST_VOLUME_IS_000:
-			for (i = 0; i < l; i++)
-				parts[2][i] = '0';
-			break;
-
-		case FIRST_VOLUME_IS_001:
-			for (i = 0; i < l; i++)
-				parts[2][i] = (i < l - 1) ? '0' : '1';
-			break;
-
-		case FIRST_VOLUME_IS_RAR:
-			if (g_str_has_suffix (parts[1], "r")) {
-				parts[2][0] = 'a';
-				parts[2][1] = 'r';
-			}
-			else {
-				parts[2][0] = 'A';
-				parts[2][1] = 'R';
-			}
-			break;
-		}
-
-		volume_name = g_strjoinv ("", parts);
-
-		g_strfreev (parts);
-	}
-	g_regex_unref (re);
-
-	if (volume_name != NULL) {
-		char *tmp;
-
-		tmp = volume_name;
-		volume_name = g_filename_from_utf8 (tmp, -1, NULL, NULL, NULL);
-		g_free (tmp);
-	}
-
-	return volume_name;
-}
-
-
-static void
-check_multi_vomule (FrCommand *comm)
-{
-	GFile *file;
-	char   buffer[11];
-
-	file = g_file_new_for_path (comm->filename);
-	if (! g_load_file_in_buffer (file, buffer, 11, NULL)) {
-		g_object_unref (file);
-		return;
-	}
-
-	if ((buffer[10] & 0x01) == 0x01) {
-		char   *volume_name = NULL;
-		char   *name;
-
-		name = g_filename_to_utf8 (file_name_from_path (comm->filename), -1, NULL, NULL, NULL);
-
-		volume_name = get_first_volume_name (name, "^(.*\\.part)([0-9]+)(\\.rar)$", FIRST_VOLUME_IS_001);
-		if (volume_name == NULL)
-			volume_name = get_first_volume_name (name, "^(.*\\.r)([0-9]+)$", FIRST_VOLUME_IS_RAR);
-		if (volume_name == NULL)
-			volume_name = get_first_volume_name (name, "^(.*\\.)([0-9]+)$", FIRST_VOLUME_IS_001);
-
-		if (volume_name != NULL) {
-			GFile *parent;
-			GFile *child;
-			char  *volume_filename;
-
-			parent = g_file_get_parent (file);
-			child = g_file_get_child (parent, volume_name);
-			volume_filename = g_file_get_path (child);
-			fr_command_set_multi_volume (comm, volume_filename);
-
-			g_free (volume_filename);
-			g_object_unref (child);
-			g_object_unref (parent);
-		}
-
-		g_free (name);
-	}
-
-	g_object_unref (file);
-}
-
-
 static void
 list__begin (gpointer data)
 {
@@ -433,7 +328,7 @@ list__begin (gpointer data)
 static void
 fr_command_rar_list (FrCommand  *comm)
 {
-	check_multi_vomule (comm);
+	rar_check_multi_volume (comm);
 
 	fr_process_set_out_line_func (comm->process, process_line, comm);
 
@@ -458,43 +353,14 @@ fr_command_rar_list (FrCommand  *comm)
 }
 
 
-static char Progress_Message[4196];
-static char Progress_Filename[4096];
-
-
 static void
 parse_progress_line (FrCommand  *comm,
 		     const char *prefix,
 		     const char *message_prefix,
 		     const char *line)
 {
-	int prefix_len;
-
-	prefix_len = strlen (prefix);
-	if (strncmp (line, prefix, prefix_len) == 0) {
-		double  fraction;
-		int     len;
-		char   *b_idx;
-
-		strcpy (Progress_Filename, line + prefix_len);
-
-		/* when a new volume is created a sequence of backspaces is
-		 * issued, remove the backspaces from the filename */
-		b_idx = strchr (Progress_Filename, '\x08');
-		if (b_idx != NULL)
-			*b_idx = 0;
-
-		/* remove the OK at the end of the filename */
-		len = strlen (Progress_Filename);
-		if ((len > 5) && (strncmp (Progress_Filename + len - 5, "  OK ", 5) == 0))
-			Progress_Filename[len - 5] = 0;
-
-		sprintf (Progress_Message, "%s%s", message_prefix, file_name_from_path (Progress_Filename));
-		fr_command_message (comm, Progress_Message);
-
-		fraction = (double) ++comm->n_file / (comm->n_files + 1);
-		fr_command_progress (comm, fraction);
-	}
+	if (strncmp (line, prefix, strlen (prefix)) == 0)
+		fr_command_progress (comm, (double) ++comm->n_file / (comm->n_files + 1));
 }
 
 
@@ -512,11 +378,15 @@ process_line__add (char     *line,
 		if ((comm->volume_size > 0)
 		    && g_regex_match_simple ("^.*\\.part(0)*2\\.rar$", uri, G_REGEX_CASELESS, 0))
 		{
-			char *volume_filename;
+			char  *volume_filename;
+			GFile *volume_file;
 
 			volume_filename = g_strdup (archive_filename);
 			volume_filename[strlen (volume_filename) - 5] = '1';
-			fr_command_set_multi_volume (comm, volume_filename);
+			volume_file = g_file_new_for_path (volume_filename);
+			fr_command_set_multi_volume (comm, volume_file);
+
+			g_object_unref (volume_file);
 			g_free (volume_filename);
 		}
 		fr_command_working_archive (comm, uri);
@@ -805,14 +675,14 @@ const char *rar_mime_type[] = { "application/x-cbr",
 				NULL };
 
 
-const char **
+static const char **
 fr_command_rar_get_mime_types (FrCommand *comm)
 {
 	return rar_mime_type;
 }
 
 
-FrCommandCap
+static FrCommandCap
 fr_command_rar_get_capabilities (FrCommand  *comm,
 			         const char *mime_type,
 				 gboolean    check_command)
