@@ -22,10 +22,15 @@
 
 #include <config.h>
 #include <string.h>
+#include <gtk/gtk.h>
 #include "gtk-utils.h"
 
 #define LOAD_BUFFER_SIZE 65536
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+#define gtk_vbox_new(X, Y) gtk_box_new(GTK_ORIENTATION_VERTICAL, Y)
+#define gtk_hbox_new(X, Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL, Y)
+#endif
 
 static void
 count_selected (GtkTreeModel *model,
@@ -74,17 +79,9 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 	if (stock_id == NULL)
 		stock_id = GTK_STOCK_DIALOG_INFO;
 
-	dialog = gtk_dialog_new_with_buttons ("", parent, flags, NULL);
-
+	dialog = gtk_dialog_new_with_buttons ("", parent, flags, NULL, NULL);
 	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
-#if !GTK_CHECK_VERSION(2,22,0)
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-#endif
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
-	gtk_container_set_border_width (GTK_CONTAINER (content_area), 6);
-	gtk_box_set_spacing (GTK_BOX (content_area), 8);
-
 	/* Add label and image */
 
 	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
@@ -212,52 +209,51 @@ _gtk_request_dialog_run (GtkWindow        *parent,
 
 	stock_id = GTK_STOCK_DIALOG_QUESTION;
 
-	dialog = gtk_dialog_new_with_buttons (title, parent, flags, NULL);
-
+	dialog = gtk_dialog_new_with_buttons (title, parent, flags, NULL, NULL);
 	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
-#if !GTK_CHECK_VERSION(2,22,0)
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-#endif
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
-	gtk_container_set_border_width (GTK_CONTAINER (content_area), 6);
-	gtk_box_set_spacing (GTK_BOX (content_area), 8);
-
 	/* Add label and image */
 
 	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+	gtk_widget_set_valign (image, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+#endif
 
-	label = gtk_label_new (message);
+	label = gtk_label_new_with_mnemonic (message);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_widget_set_valign (label, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
+#endif
 
 	entry = gtk_entry_new ();
-	gtk_widget_set_size_request (entry, 250, -1);
+	gtk_entry_set_width_chars (GTK_ENTRY (entry), 50);
 	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
 	gtk_entry_set_max_length (GTK_ENTRY (entry), max_length);
 	gtk_entry_set_text (GTK_ENTRY (entry), default_value);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
 
-	hbox = gtk_hbox_new (FALSE, 24);
-	vbox = gtk_vbox_new (FALSE, 6);
+	hbox = gtk_hbox_new (FALSE, 12);
+	vbox = gtk_vbox_new (FALSE, 12);
 
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-	gtk_box_pack_start (GTK_BOX (hbox), image,
-			    FALSE, FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (hbox), vbox,
-			    TRUE, TRUE, 0);
+	gtk_box_set_spacing (GTK_BOX (content_area), 14); /* 14 + 2 * 5 = 24 */
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+	gtk_box_set_spacing (GTK_BOX (vbox), 6);
 
-	gtk_box_pack_start (GTK_BOX (vbox), label,
-			    FALSE, FALSE, 0);
-
-	gtk_box_pack_start (GTK_BOX (vbox), entry,
-			    FALSE, FALSE, 0);
-
-	gtk_box_pack_start (GTK_BOX (content_area),
-			    hbox,
-			    FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (content_area), hbox, FALSE, FALSE, 0);
 
 	gtk_widget_show_all (hbox);
 
@@ -305,18 +301,11 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 	GtkWidget    *content_area;
 	char         *stock_id = GTK_STOCK_DIALOG_WARNING;
 
-	d = gtk_dialog_new_with_buttons ("", parent, flags, NULL);
+	d = gtk_dialog_new_with_buttons ("", parent, flags, NULL, NULL);
 	gtk_window_set_resizable (GTK_WINDOW (d), FALSE);
 
 	content_area = gtk_dialog_get_content_area (GTK_DIALOG (d));
 
-#if !GTK_CHECK_VERSION(2,22,0)
-	gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
-#endif
-	gtk_container_set_border_width (GTK_CONTAINER (d), 6);
-	gtk_container_set_border_width (GTK_CONTAINER (content_area), 6);
-	gtk_box_set_spacing (GTK_BOX (content_area), 8);
-
 	/* Add label and image */
 
 	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
@@ -380,6 +369,7 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	GtkWidget     *scrolled = NULL;
 	GtkWidget     *expander;
 	GtkWidget     *content_area;
+	GtkWidget     *action_area;
 	GtkTextBuffer *text_buf;
 	GtkTextIter    iter;
 	char          *stock_id;
@@ -400,23 +390,27 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 
 	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
-#if !GTK_CHECK_VERSION(2,22,0)
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-#endif
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
-	gtk_container_set_border_width (GTK_CONTAINER (content_area), 6);
-	gtk_box_set_spacing (GTK_BOX (content_area), 8);
-
-	gtk_widget_set_size_request (dialog, 500, -1);
+	action_area = gtk_dialog_get_action_area (GTK_DIALOG (dialog));
 
 	/* Add label and image */
 
 	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+	gtk_widget_set_valign (image, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+#endif
 
 	label = gtk_label_new ("");
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_widget_set_valign (label, GTK_ALIGN_START);
+#else
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
+#endif
 
 	escaped_message = g_markup_escape_text (primary_text, -1);
 	if (secondary_text != NULL) {
@@ -428,7 +422,7 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 		va_end (args);
 		escaped_secondary_message = g_markup_escape_text (secondary_message, -1);
 
-		markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n",
+		markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
 					       escaped_message,
 					       escaped_secondary_message);
 
@@ -442,6 +436,8 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	g_free (escaped_message);
 
 	if (view_output) {
+		gtk_widget_set_size_request (dialog, 500, -1);
+
 		/* Expander */
 
 		expander = gtk_expander_new_with_mnemonic (_("Command _Line Output"));
@@ -483,7 +479,6 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	}
 
 	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
@@ -496,9 +491,14 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 		gtk_box_pack_start (GTK_BOX (vbox), expander, TRUE, TRUE, 0);
 	}
 
-	gtk_box_pack_start (GTK_BOX (content_area),
-			    vbox,
-			    FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (content_area), vbox, FALSE, FALSE, 0);
+
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+	gtk_box_set_spacing (GTK_BOX (content_area), 14); /* 14 + 2 * 5 = 24 */
+	gtk_container_set_border_width (GTK_CONTAINER (action_area), 5);
+	gtk_box_set_spacing (GTK_BOX (action_area), 6);
+
 	gtk_widget_show_all (vbox);
 
 	return dialog;
@@ -740,7 +740,6 @@ get_mime_type_pixbuf (const char   *mime_type,
 	return pixbuf;
 }
 
-
 int
 get_folder_pixbuf_size_for_list (GtkWidget *widget)
 {
@@ -752,7 +751,6 @@ get_folder_pixbuf_size_for_list (GtkWidget *widget)
 	return MAX (icon_width, icon_height);
 }
 
-
 gboolean
 show_uri (GdkScreen   *screen,
 	  const char  *uri,
@@ -774,7 +772,7 @@ show_help_dialog (GtkWindow  *parent,
 	char   *uri;
 	GError *error = NULL;
 
-	uri = g_strconcat ("help:engrampa", section ? "?" : NULL, section, NULL);
+	uri = g_strconcat ("help:engrampa", section ? "/" : NULL, section, NULL);
 	if (! show_uri (gtk_window_get_screen (parent), uri, GDK_CURRENT_TIME, &error)) {
   		GtkWidget *dialog;
 
@@ -826,3 +824,14 @@ _gtk_builder_get_widget (GtkBuilder *builder,
 {
 	return (GtkWidget *) gtk_builder_get_object (builder, name);
 }
+
+int
+_gtk_widget_lookup_for_size (GtkWidget *widget,
+                             GtkIconSize icon_size)
+{
+	int w, h;
+	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
+					   icon_size,
+					   &w, &h);
+	return MAX (w, h);
+}
