@@ -34,15 +34,13 @@ static GtkActionEntry action_entries[] = {
 	{ "ViewMenu", NULL, N_("_View") },
 	{ "HelpMenu", NULL, N_("_Help") },
 	{ "ArrangeFilesMenu", NULL, N_("_Arrange Files") },
-	/* Translators: this is the label for the "open recent file" sub-menu. */
-	{ "OpenRecentMenu", NULL, N_("Open _Recent") },
 
 	{ "About", GTK_STOCK_ABOUT,
 	  NULL, NULL,
 	  N_("Information about the program"),
 	  G_CALLBACK (activate_action_about) },
 	{ "AddFiles", FR_STOCK_ADD_FILES,
-	  N_("_Add Files..."), NULL,
+	  N_("_Add Files…"), NULL,
 	  N_("Add files to the archive"),
 	  G_CALLBACK (activate_action_add_files) },
 	{ "AddFiles_Toolbar", FR_STOCK_ADD_FILES,
@@ -50,7 +48,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Add files to the archive"),
 	  G_CALLBACK (activate_action_add_files) },
 	{ "AddFolder", FR_STOCK_ADD_FOLDER,
-	  N_("Add a _Folder..."), NULL,
+	  N_("Add a _Folder…"), NULL,
 	  N_("Add a folder to the archive"),
 	  G_CALLBACK (activate_action_add_folder) },
 	{ "AddFolder_Toolbar", FR_STOCK_ADD_FOLDER,
@@ -79,7 +77,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Paste the clipboard"),
 	  G_CALLBACK (activate_action_paste) },
 	{ "Rename", NULL,
-	  N_("_Rename..."), "F2",
+	  N_("_Rename…"), "F2",
 	  N_("Rename the selection"),
 	  G_CALLBACK (activate_action_rename) },
 	{ "Delete", GTK_STOCK_DELETE,
@@ -100,7 +98,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Paste the clipboard"),
 	  G_CALLBACK (activate_action_paste_folder_from_sidebar) },
 	{ "RenameFolderFromSidebar", NULL,
-	  N_("_Rename..."), "F2",
+	  N_("_Rename…"), "F2",
 	  N_("Rename the selection"),
 	  G_CALLBACK (activate_action_rename_folder_from_sidebar) },
 	{ "DeleteFolderFromSidebar", GTK_STOCK_DELETE,
@@ -113,11 +111,11 @@ static GtkActionEntry action_entries[] = {
 	  N_("Deselect all files"),
 	  G_CALLBACK (activate_action_deselect_all) },
 	{ "Extract", FR_STOCK_EXTRACT,
-	  N_("_Extract..."), "<control>E",
+	  N_("_Extract…"), "<control>E",
 	  N_("Extract files from the archive"),
 	  G_CALLBACK (activate_action_extract) },
 	{ "ExtractFolderFromSidebar", FR_STOCK_EXTRACT,
-	  N_("_Extract..."), NULL,
+	  N_("_Extract…"), NULL,
 	  N_("Extract files from the archive"),
 	  G_CALLBACK (activate_action_extract_folder_from_sidebar) },
 	{ "Extract_Toolbar", FR_STOCK_EXTRACT,
@@ -125,7 +123,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Extract files from the archive"),
 	  G_CALLBACK (activate_action_extract) },
 	{ "Find", GTK_STOCK_FIND,
-	  N_("Find..."), NULL,
+	  N_("Find…"), NULL,
 	  NULL,
 	  G_CALLBACK (activate_action_find) },
 
@@ -134,11 +132,11 @@ static GtkActionEntry action_entries[] = {
 	  N_("View the output produced by the last executed command"),
 	  G_CALLBACK (activate_action_last_output) },
 	{ "New", GTK_STOCK_NEW,
-	  N_("New..."), NULL,
+	  N_("New…"), NULL,
 	  N_("Create a new archive"),
 	  G_CALLBACK (activate_action_new) },
 	{ "Open", GTK_STOCK_OPEN,
-	  N_("Open..."), NULL,
+	  N_("Open…"), NULL,
 	  N_("Open archive"),
 	  G_CALLBACK (activate_action_open) },
 	{ "Open_Toolbar", GTK_STOCK_OPEN,
@@ -146,11 +144,11 @@ static GtkActionEntry action_entries[] = {
 	  N_("Open archive"),
 	  G_CALLBACK (activate_action_open) },
 	{ "OpenSelection", NULL,
-	  N_("_Open With..."), NULL,
+	  N_("_Open With…"), NULL,
 	  N_("Open selected files with an application"),
 	  G_CALLBACK (activate_action_open_with) },
 	{ "Password", NULL,
-	  N_("Pass_word..."), NULL,
+	  N_("Pass_word…"), NULL,
 	  N_("Specify a password for this archive"),
 	  G_CALLBACK (activate_action_password) },
 	{ "Properties", GTK_STOCK_PROPERTIES,
@@ -162,7 +160,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Reload current archive"),
 	  G_CALLBACK (activate_action_reload) },
 	{ "SaveAs", GTK_STOCK_SAVE_AS,
-	  N_("Save As..."), NULL,
+	  N_("Save As…"), NULL,
 	  N_("Save the current archive with a different name"),
 	  G_CALLBACK (activate_action_save_as) },
 	{ "SelectAll", GTK_STOCK_SELECT_ALL,
@@ -279,9 +277,7 @@ static const gchar *ui_info =
 "    <menu name='Archive' action='FileMenu'>"
 "      <menuitem action='New'/>"
 "      <menuitem action='Open'/>"
-"      <menu name='OpenRecentMenu' action='OpenRecentMenu'>"
-"        <menuitem action='Open'/>"
-"      </menu>"
+"      <menuitem action='OpenRecent'/>"
 "      <menuitem action='SaveAs'/>"
 "      <separator/>"
 "      <menuitem action='Extract'/>"
@@ -338,6 +334,7 @@ static const gchar *ui_info =
 "  </menubar>"
 "  <toolbar name='ToolBar'>"
 "    <toolitem action='New'/>"
+"    <toolitem action='OpenRecent_Toolbar'/>"
 "    <separator/>"
 "    <toolitem action='Extract_Toolbar'/>"
 "    <separator/>"
